import logging

from abb.core import env
from rdflib.namespace import Namespace
from rdflib.term import bind, Literal, URIRef
from sparqltools.query import QueryBuilder


_registered_namespaces = {}


def get_default_namespace():
    return Namespace(env('NAMESPACE'))


def register_namespace(prefix, namespace_name):
    ns = Namespace(namespace_name)
    global _registered_namespaces
    if prefix in _registered_namespaces:
        if str(_registered_namespaces[prefix]) == namespace_name:
            return
        raise KeyError("Namespace with prefix '%s' already registered." % prefix)
    _registered_namespaces[prefix] = ns


def get_all_namespaces():
    global _registered_namespaces
    _registered_namespaces[""] = get_default_namespace()
    return _registered_namespaces.items()


def mailto(email):
    email = email.lower().strip()
    ref = "mailto:%s" % email
    return URIRef(ref)


class SPARQL(QueryBuilder):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for prefix, ns in get_all_namespaces():
            self.set_prefix(prefix, ns)


def bind_datatypes(types, ns=None):
    if ns is None:
        ns = get_default_namespace()

    for dtype in types:
        name = dtype.__name__
        uri = ns[name]
        logging.debug("Binding datatype (%s): %s" % (name, uri))
        bind(uri, dtype)


if __name__ == '__main__':
    assert Literal
    assert URIRef
